/* -------------------------------------------------------------------- */
/* SMS Client, send messages to mobile phones and pagers		*/
/*									*/
/* libertel.c								*/
/*									*/
/*  Copyright (C) 1998 Aart Koelewijn					*/
/*									*/
/*  This library is free software; you can redistribute it and/or	*/
/*  modify it under the terms of the GNU Library General Public		*/
/*  License as published by the Free Software Foundation; either	*/
/*  version 2 of the License, or (at your option) any later version.	*/
/*									*/
/*  This library is distributed in the hope that it will be useful,	*/
/*  but WITHOUT ANY WARRANTY; without even the implied warranty of	*/
/*  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU	*/
/*  Library General Public License for more details.			*/
/*									*/
/*  You should have received a copy of the GNU Library General Public	*/
/*  License along with this library; if not, write to the Free		*/
/*  Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.	*/
/*									*/
/*  You can contact the author at this e-mail address:			*/
/*									*/
/*  aart@mtack.xs4all.nl						*/
/*									*/
/* -------------------------------------------------------------------- */
/* $Id: libertel.c,v 0.1 1998/04/02 22:49:15 aart
   -------------------------------------------------------------------- */

#include <stdio.h>
#include <termios.h>
#include <strings.h>

#include <dial/modems.h>

#include "logfile.h"
#include "driver.h"
#include "expect.h"
#include "sms_modem.h"
#include "sms_error.h"
#include "sms_list.h"
#include "parserc.h"

/* -------------------------------------------------------------------- */

static char ACK1[] = "Welkom bij LIBERTEL Text-PC\r\n"
                     "Als u vragen heeft over deze dienst kunt u bellen naar 06 54 500 100\r\n";
static char ACK2[] = "Typ het mobiele telefoonnummer in volgens het internationale formaat, \r\n"
		     "bijvoorbeeld 31654XXXXXX, gevolgd door enter (of druk op enter om te stoppen)\r\n"
	             ">";
	             
static char ACK3[] = "Typ uw bericht in, (maximaal 160 tekens), gevolgd door enter\r\n"
		     ">";

static char ACK4[] = "Bericht geaccepteerd\r\n";
		     
/* -------------------------------------------------------------------- */

static int 	fd,
		fd_in,
		fd_out;

		
#define LONGTIMEOUT 	30
#define MAX_BUFSIZE	1024
#define TIMEOUT 	5

/* -------------------------------------------------------------------- */

static int LIBERTEL_dial(char *number, char *service);
static void LIBERTEL_hangup(void);
static int LIBERTEL_sendmessage(char *msisdn, char *message);
	
/* -------------------------------------------------------------------- */
/* -------------------------------------------------------------------- */
static int LIBERTEL_dial(char *number, char *service)
{
	lprintf(LOG_STANDARD, "Dialing %s SMSC %s...\n", service, number);

	fd = SMS_dial(number, service);
	if (fd < 0)
	{
		mdmperror("Dialing");
		return EDIAL;
	}
	
	fd_in  = fd;
	fd_out = fd;

	lprintf(LOG_STANDARD, "Connection Established.\n");

	return 0;
}


/* -------------------------------------------------------------------- */
/* -------------------------------------------------------------------- */
static void LIBERTEL_hangup(void)
{
	lprintf(LOG_STANDARD, "Hangup...\n");	
	hangup(fd);
}


/* -------------------------------------------------------------------- */
/* -------------------------------------------------------------------- */
static int LIBERTEL_sendmessage(char *msisdn, char *message)
{
	char buf[1024];
	char nmessage[1024];


	if (expstr(fd_in, buf, ACK1, MAX_BUFSIZE, TIMEOUT) == 0)
	{
		lprintf(LOG_STANDARD, "Libertel Service Login\n");
	}
	else
	{
		lprintf(LOG_STANDARD, "No Libertel Service Response\n");
		
		LIBERTEL_hangup();
		return ELIBERTEL_NORESPONSE;
	}



	if (expstr(fd_in, buf, ACK2, MAX_BUFSIZE, TIMEOUT) == 0)
	{
		lprintf(LOG_STANDARD, "Received Number Request\n");
	}
	else
	{
		lprintf(LOG_STANDARD, "No Number Request\n");
		
		LIBERTEL_hangup();
		return ELIBERTEL_NONUMBER;
	}



	twrite(fd_out, msisdn, strlen(msisdn), TIMEOUT);
	twrite(fd_out, "\r\n", strlen("\r\n"), TIMEOUT);



	if (expstr(fd_in, buf, ACK3, MAX_BUFSIZE, TIMEOUT) == 0)
	{
		lprintf(LOG_STANDARD, "Received Message Request\n");
	}
	else
	{
		lprintf(LOG_STANDARD, "No Message Request\n");
		
		LIBERTEL_hangup();
		return ELIBERTEL_NOMESSAGE;
	}



	strcpy(nmessage, message);
	strcat(nmessage, "\r\n");

	twrite(fd_out, nmessage, strlen(nmessage), TIMEOUT);

	
	if (expstr(fd_in, buf, ACK4, MAX_BUFSIZE, TIMEOUT) == 0)
	{
		lprintf(LOG_STANDARD, "Received Message Delivery Response\n");
	}
	else
	{
		lprintf(LOG_STANDARD, "No Message Delivery Response\n");
		
		LIBERTEL_hangup();
		return ELIBERTEL_NODELIVERY;
	}


	twrite(fd_out, "\r\n", strlen("\r\n"), TIMEOUT);

	return 0;
}

/* -------------------------------------------------------------------- */
/* -------------------------------------------------------------------- */
void libertel_deliver(SMS_list *node, char *message)
{
	char 	*msisdn, 
		*smsc_number,
		*name, 
		*mservice;

	int 	error;
	

	msisdn      = get_number(node);
	name        = get_name(node);
	mservice    = get_service(node);
	smsc_number = get_smscnumber(mservice);

	lprintf(LOG_VERBOSE, "Name     :%s\n", name);
	lprintf(LOG_VERBOSE, "Number   :%s\n", msisdn);


 	error = LIBERTEL_dial(smsc_number, mservice);
	if (error)
	{
		set_delivery(node, error);
		return;
	}

	error = LIBERTEL_sendmessage(msisdn, message);
	if (error)
	{
		set_delivery(node, error);
		return;
	}

	LIBERTEL_hangup();
}


/* -------------------------------------------------------------------- */
/* -------------------------------------------------------------------- */
SMS_list *libertel_main(SMS_list *list, char *message)
{
	SMS_list 
		*node;

	lprintf(LOG_VERBOSE, "%s driver called\n", libertel_device.name);
	
	node = get_first(list);
	while (node != NULL)
	{
		set_delivery(node, 0);
		libertel_deliver(node, message);
		node = get_next(node);
	}

	return NULL; 
}


/* -------------------------------------------------------------------- */
/* -------------------------------------------------------------------- */
int libertel_validate_id(char *id)
{
	return is_numeric(id);
}


/* -------------------------------------------------------------------- */
/* -------------------------------------------------------------------- */
DEVICE_ENTRY libertel_device = {
	"LIBERTEL",
	libertel_main,
	libertel_validate_id
};

